# messagecheck.tcl --
# 
# Part of: MHTCL
# Contents: example script to check a signed file
# Date: Wed Jul  2, 2003
# 
# Abstract
# 
# 
# 
# Copyright (c) 2003 Marco Maggi
# 
# This is free  software you can redistribute it  and/or modify it under
# the terms of  the GNU General Public License as  published by the Free
# Software Foundation; either  version 2, or (at your  option) any later
# version.
# 
# This  file is  distributed in  the hope  that it  will be  useful, but
# WITHOUT   ANY  WARRANTY;  without   even  the   implied  warranty   of
# MERCHANTABILITY  or FITNESS  FOR A  PARTICULAR PURPOSE.   See  the GNU
# General Public License for more details.
# 
# You  should have received  a copy  of the  GNU General  Public License
# along with this file; see the file COPYING.  If not, write to the Free
# Software Foundation,  Inc., 59  Temple Place -  Suite 330,  Boston, MA
# 02111-1307, USA.
# 

package require Tcl
package require mhash

proc main { argc argv } {
    if { $argc != 2 } {
	puts stderr "usage: messagecheck.tcl filename passphrase"
	exit 1
    }

    foreach {filename passphrase} $argv {}

    if { ! [file isfile $filename] } {
	puts stderr "unexistent message file"
	exit 1
    }
    if { [read_message $filename message] } {
	puts stderr "error reading file"
	exit 1
    }
    if { [split_message $message text signature] } {
	puts stderr "format error in signed file"
	exit 1
    }
    if { [parse_signature $signature algorithm authkey] } {
	puts stderr "format error in signature"
	exit 1
    }
    if { ([string length $algorithm] == 0) || ([string length $authkey] == 0) } {
	puts stderr "bad signature"
	exit 1
    }
    if { ! [mhash info hash exists $algorithm] } {
	puts stderr "unknown hash algorithm \"$algorithm\""
	exit 1
    }

    set thread [mhash hmac $algorithm $passphrase]
    puts -nonewline $thread $text
    set crypto [fconfigure $thread -digest]
    close $thread
    binary scan $crypto H[expr {[string length $crypto]*2}] crypto

    if { [string equal $authkey $crypto] } {
	puts "correct authentication"
    } else {
	puts "authentication failure"
    }

    exit 0
}

proc read_message { filename msgvar } {
    upvar	$msgvar message
    return [catch {
	set id [open $filename { RDONLY }]
	set message [read $id]
	close $id
    } res]
}

proc split_message { message textvar signvar } {
    upvar	$textvar text $signvar sign
    set rex "^(.+)\n-- BEGIN HMAC SIGNATURE --\n(.+)\n+-- END HMAC SIGNATURE --\n+$"

    if { ! [regexp -- $rex $message dummy text sign] } {
	return 1
    }
    return 0
}

proc parse_signature { signature algovar keyvar } {
    upvar	$algovar algorithm $keyvar crypto
    set rex	{^([^:]+):\s*(.+)$}
    set algorithm {}
    set crypto {}

    foreach line [split $signature \n] {
	set line [string trim $line]
	if { [string length $line] == 0 } { continue }
	if { ! [regexp -- $rex $line dummy name value] } {
	    return 1
	}
	switch -exact -- $name {
	    Algorithm	{ set algorithm $value }
	    Key		{ set crypto    $value }
	}
    }
    return 0
}

main $argc $argv

### end of file
# Local Variables:
# mode: tcl
# page-delimiter: "^#PAGE"
# End:
