# messagesign.tcl --
# 
# Part of: MHTCL
# Contents: example script to sign a file
# Date: Wed Jul  2, 2003
# 
# Abstract
# 
# 
# 
# Copyright (c) 2003 Marco Maggi
# 
# This is free  software you can redistribute it  and/or modify it under
# the terms of  the GNU General Public License as  published by the Free
# Software Foundation; either  version 2, or (at your  option) any later
# version.
# 
# This  file is  distributed in  the hope  that it  will be  useful, but
# WITHOUT   ANY  WARRANTY;  without   even  the   implied  warranty   of
# MERCHANTABILITY  or FITNESS  FOR A  PARTICULAR PURPOSE.   See  the GNU
# General Public License for more details.
# 
# You  should have received  a copy  of the  GNU General  Public License
# along with this file; see the file COPYING.  If not, write to the Free
# Software Foundation,  Inc., 59  Temple Place -  Suite 330,  Boston, MA
# 02111-1307, USA.
# 

package require Tcl
package require mhash

proc main { argc argv } {
    if { $argc != 3 } {
	puts stderr "usage: messagesign.tcl filename algorithm passphrase"
	exit 1
    }

    foreach {filename algorithm passphrase} $argv {}

    if { ! [mhash info hash exists $algorithm] } {
	puts stderr "unknown hash algorithm \"$algorithm\""
	exit 1
    }
    if { ! [mhash info hash ishmacalgo $algorithm] } {
	puts stderr "algorithm \"$algorithm\" can't be used for HMAC"
	exit 1
    }
    if { ! [file isfile $filename] } {
	puts stderr "unexistent message file"
	exit 1
    }

    read_message $filename message

    set thread [mhash hmac $algorithm $passphrase]
    puts -nonewline $thread $message
    set crypto [fconfigure $thread -digest]
    close $thread
    binary scan $crypto H[expr {[string length $crypto]*2}] crypto

    write_message $filename $message $algorithm $crypto
    exit 0
}

proc read_message { filename varname } {
    upvar	$varname message

    set id [open $filename { RDONLY }]
    set message [read $id]
    close $id
    return
}

proc write_message { filename message algorithm crypto } {
    append sigptn \
	    "\n-- BEGIN HMAC SIGNATURE --\n" \
	    "Algorithm: %s\n" \
	    "Key: %s\n" \
	    "-- END HMAC SIGNATURE --\n"

    set filename  [format "%s.sig" $filename]
    set signature [format $sigptn $algorithm $crypto]
    append message $signature

    set id [open $filename { CREAT WRONLY TRUNC }]
    puts $id $message
    close $id
    return
}

main $argc $argv

### end of file
# Local Variables:
# mode: tcl
# page-delimiter: "^#PAGE"
# End:
