# passwdadd.tcl --
# 
# Part of: MHTCL
# Contents: example script for passwd file generation
# Date: Wed Jul  2, 2003
# 
# Abstract
# 
# 
# 
# Copyright (c) 2003 Marco Maggi
# 
# This is free  software you can redistribute it  and/or modify it under
# the terms of  the GNU General Public License as  published by the Free
# Software Foundation; either  version 2, or (at your  option) any later
# version.
# 
# This  file is  distributed in  the hope  that it  will be  useful, but
# WITHOUT   ANY  WARRANTY;  without   even  the   implied  warranty   of
# MERCHANTABILITY  or FITNESS  FOR A  PARTICULAR PURPOSE.   See  the GNU
# General Public License for more details.
# 
# You  should have received  a copy  of the  GNU General  Public License
# along with this file; see the file COPYING.  If not, write to the Free
# Software Foundation,  Inc., 59  Temple Place -  Suite 330,  Boston, MA
# 02111-1307, USA.
# 

package require Tcl
package require mhash

proc main { argc argv } {
    if { $argc != 4 } {
	puts stderr "usage: passwdadd.tcl filename algorithm username password"
	exit 1
    }

    foreach { filename algorithm name password } $argv {}

    if { ! [mhash info hash exists $algorithm] } {
	puts stderr "unknown hash algorithm \"$algorithm\""
	exit 1
    }

    read_passwd $filename users

    set thread [mhash hash $algorithm]
    puts -nonewline $thread $password
    set crypto [fconfigure $thread -digest]
    close $thread

    binary scan $crypto H[expr {[string length $crypto]*2}] crypto
    set users($name) $crypto

    write_passwd $filename users
    exit 0
}

proc read_passwd { filename varname } {
    upvar	$varname users

    if { ! [file isfile $filename] } {
	array set users {}
	return
    }

    set id [open $filename { RDONLY }]
    while { ! [eof $id] } {
	gets $id line
	set line [string trim $line]
	if { ! [string length $line] } { continue }
	if { ! [regexp -- {^([^:]+):(.+)$} $line dummy name crypto] } {
	    puts stderr "format error in passwd file"
	    close $id
	    exit 1
	}
	set users($name) $crypto
    }
    close $id
    return
}

proc write_passwd { filename varname } {
    upvar	$varname users

    set id [open $filename { CREAT WRONLY TRUNC }]
    foreach name [lsort [array names users]] {
	puts $id [format "%s:%s" $name $users($name)]
    }
    close $id
    return
}

main $argc $argv

### end of file
# Local Variables:
# mode: tcl
# page-delimiter: "^#PAGE"
# End:
