/* init.c --
   
   Part of: MHTCL
   Contents: library initialisation functions
   Date: Sun Nov  3, 2002
   
   Abstract
   
	This file defines the initialisation functions.
   
   Copyright (c) 2002, 2003, 2004 Marco Maggi
   
   This  is free  software; you  can redistribute  it and/or  modify it
   under  the  terms  of  the  GNU Lesser  General  Public  License  as
   published by the Free Software Foundation; either version 2.1 of the
   License, or (at your option) any later version.
   
   This library is  distributed in the hope that it  will be useful, but
   WITHOUT  ANY   WARRANTY;  without   even  the  implied   warranty  of
   MERCHANTABILITY  or FITNESS FOR  A PARTICULAR  PURPOSE.  See  the GNU
   Lesser General Public License for more details.
   
   You  should have received  a copy  of the  GNU Lesser  General Public
   License along with  this library; if not, write  to the Free Software
   Foundation, Inc.,  59 Temple Place, Suite 330,  Boston, MA 02111-1307
   USA
*/


/** ------------------------------------------------------------
 ** Header files.
 ** ----------------------------------------------------------*/

#include "mhtclInt.h"


/* Mhtcl_Init --

	Package initialisation.

   Arguments:

	interp -	the interp in which register the package

   Results:

        Returns TCL_OK or TCL_ERROR.

*/

EXTERN int
Mhtcl_Init (Tcl_Interp *interp)
{
  int		e;
  

#ifdef USE_TCL_STUBS
#warning using tcl stub
  if (Tcl_InitStubs(interp, "8.4", 0) == NULL) {
    return TCL_ERROR;
  }
#endif

#ifdef USE_TCLMORE_STUB
#warning using more stub
  if (More_InitStub(interp, TCLMORE_QUOTED_VERSION, 0) == NULL) {
    return TCL_ERROR;
  }
#endif

  e = Tcl_PkgProvideEx(interp, TCL_QUOTED_PACKAGE_NAME,
		       TCL_QUOTED_PACKAGE_VERSION,
		       (ClientData) &mhtclStub);
  return ((TCL_OK == e)? Mhtcl_CreateCommands(interp) : e);
}

/* Mhtcl_SafeInit --

	Initialises the package in a safe interp.

   Arguments:

	interp -	the interp in which register the package

   Results:

        Returns TCL_OK or TCL_ERROR.

*/

EXTERN int
Mhtcl_SafeInit (Tcl_Interp *interp)
{
  return Mhtcl_Init(interp);
}

/* end of file */
