/* mhtclDecls.h --
   
   Part of: MHTCL
   Contents: declarations for the stubs
   Date: Sun Aug 10, 2003
   
   Abstract
   
	Declarations for both the internal and external header files.
   
   Copyright (c) 2003, 2004 Marco Maggi
   
   This is free software; you  can redistribute it and/or modify it under
   the terms of the GNU Lesser General Public License as published by the
   Free Software  Foundation; either version  2.1 of the License,  or (at
   your option) any later version.
   
   This library  is distributed in the  hope that it will  be useful, but
   WITHOUT   ANY  WARRANTY;   without  even   the  implied   warranty  of
   MERCHANTABILITY  or FITNESS  FOR A  PARTICULAR PURPOSE.   See  the GNU
   Lesser General Public License for more details.
   
   You  should have  received a  copy of  the GNU  Lesser  General Public
   License along  with this library; if  not, write to  the Free Software
   Foundation, Inc.,  59 Temple Place,  Suite 330, Boston,  MA 02111-1307
   USA
*/

#ifndef __MHTCLDECLS_H
#define __MHTCLDECLS_H 1


/** ------------------------------------------------------------
 ** Algorithms identifiers.
 ** ----------------------------------------------------------*/

/* do not modify the order when adding new algorithms */
#define MHTCL_HASH_NONE			255
#define MHTCL_HASH_ADLER32		0
#define MHTCL_HASH_CRC32		1
#define MHTCL_HASH_CRC32B		2
#define MHTCL_HASH_GOST			3
#define MHTCL_HASH_HAVAL128		4
#define MHTCL_HASH_HAVAL160		5
#define MHTCL_HASH_HAVAL192		6
#define MHTCL_HASH_HAVAL224		7
#define MHTCL_HASH_HAVAL256		8
#define MHTCL_HASH_MD4			9
#define MHTCL_HASH_MD5			10
#define MHTCL_HASH_RIPEMD160		11
#define MHTCL_HASH_SHA1			12
#define MHTCL_HASH_SHA256		13
#define MHTCL_HASH_TIGER		14
#define MHTCL_HASH_TIGER128		15
#define MHTCL_HASH_TIGER160		16

/* added: Sat May  1, 2004 */
#define MHTCL_HASH_RIPEMD128		17
#define MHTCL_HASH_RIPEMD256		18
#define MHTCL_HASH_RIPEMD320		19
#define MHTCL_HASH_SHA224		20
#define MHTCL_HASH_SHA384		21
#define MHTCL_HASH_SHA512		22
#define MHTCL_HASH_WHIRLPOOL		23

#define MHTCL_HASH_INVALID		24 /* do not use this */

/* do not modify the order when adding new algorithms */
#define MHTCL_KEYGEN_NONE		255
#define MHTCL_KEYGEN_ASIS		0
#define MHTCL_KEYGEN_HEX		1
#define MHTCL_KEYGEN_MCRYPT		2
#define MHTCL_KEYGEN_PKDES		3
#define MHTCL_KEYGEN_S2K_ISALTED	4
#define MHTCL_KEYGEN_S2K_SALTED		5
#define MHTCL_KEYGEN_S2K_SIMPLE		6

#define MHTCL_KEYGEN_INVALID		7 /* do not use this */


/** ------------------------------------------------------------
 ** Type declarations.
 ** ----------------------------------------------------------*/

#ifndef MHTCL_THREAD_TYPEDEF
typedef ClientData		Mhtcl_Thread;
#endif

#ifndef MORE_BLOCK_NULL_VALUE
#  define MORE_BLOCK_NULL_VALUE		{ 0, NULL }
#endif


/** ------------------------------------------------------------
 ** Prototypes.
 ** ----------------------------------------------------------*/

#ifndef USE_MHTCL_STUBS

/* mhash interface */


EXTERN More_Error Mhtcl_OpenHashThread	_ANSI_ARGS_((int algorithm,
						     Mhtcl_Thread *thread));
EXTERN More_Error Mhtcl_OpenHMACThread	_ANSI_ARGS_((int algorithm,
						     More_Block password,
						     Mhtcl_Thread *thread));

EXTERN void Mhtcl_Digest		_ANSI_ARGS_((Mhtcl_Thread thread,
						     More_Block block));

EXTERN More_Error Mhtcl_Close	_ANSI_ARGS_((Mhtcl_Thread thread,
					     Tcl_Obj ** objPtrVar));

EXTERN More_Error Mhtcl_Copy	_ANSI_ARGS_((Mhtcl_Thread src,
					     Mhtcl_Thread *dstVar));

EXTERN void Mhtcl_Abort	_ANSI_ARGS_((Mhtcl_Thread thread));


EXTERN More_Error Mhtcl_Keygen _ANSI_ARGS_((int algorithm,
					    int length,
					    More_Block password,
					    int hash1,
					    int hash2,
					    int count,
					    More_Block salt,
					    Tcl_Obj **keyVar));


/* Channel interface. */

EXTERN Tcl_Channel Mhtcl_OpenHashChannel _ANSI_ARGS_((Mhtcl_Thread thread));

EXTERN More_Error Mhtcl_MakeChannelHash _ANSI_ARGS_((Tcl_Channel channel,
						     Tcl_Obj **digestVar));

/* Miscellaneous. */

EXTERN int Mhtcl_GetHashAlgFromObj _ANSI_ARGS_((Tcl_Interp *	interp,
						Tcl_Obj *	objPtr,
						int *	algPtr));
EXTERN int Mhtcl_GetKeygenAlgFromObj _ANSI_ARGS_((Tcl_Interp *	interp,
						  Tcl_Obj *	objPtr,
						  int *algPtr));

EXTERN int Mhtcl_HashExists	_ANSI_ARGS_((int algorithm));
EXTERN int Mhtcl_KeygenExists	_ANSI_ARGS_((int algorithm));

EXTERN int Mhtcl_IsHMACAlgo	_ANSI_ARGS_((int algorithm));
EXTERN size_t Mhtcl_HashBlockSize	_ANSI_ARGS_((int algorithm));
EXTERN int Mhtcl_KeygenUsesSalt	_ANSI_ARGS_((int algorithm));
EXTERN int Mhtcl_KeygenUsesCount	_ANSI_ARGS_((int algorithm));
EXTERN int Mhtcl_KeygenSaltSize	_ANSI_ARGS_((int algorithm));
EXTERN int Mhtcl_KeygenMaxKeySize	_ANSI_ARGS_((int algorithm));
EXTERN int Mhtcl_KeygenUsesHash	_ANSI_ARGS_((int algorithm));

EXTERN Tcl_Obj * Mhtcl_ListHashAlgos _ANSI_ARGS_((void));
EXTERN Tcl_Obj * Mhtcl_ListKeygenAlgos _ANSI_ARGS_((void));

EXTERN int Mhtcl_MhashVersion	_ANSI_ARGS_((void));

#endif /* not defined USE_MHTCL_STUBS */


/** ------------------------------------------------------------
 ** Stub table typedef
 ** ----------------------------------------------------------*/

typedef struct Mhtcl_StubTable {

  /* mhash interface */


  More_Error (*mhtcl_OpenHashThread)	_ANSI_ARGS_((int algorithm,
						     Mhtcl_Thread *thread));
  More_Error (*mhtcl_OpenHMACThread)	_ANSI_ARGS_((int algorithm,
						     More_Block password,
						     Mhtcl_Thread *thread));

  void (*mhtcl_Digest) _ANSI_ARGS_((Mhtcl_Thread thread, More_Block block));

  More_Error (*mhtcl_Close)	_ANSI_ARGS_((Mhtcl_Thread thread,
					     Tcl_Obj ** objPtrVar));

  More_Error (*mhtcl_Copy)	_ANSI_ARGS_((Mhtcl_Thread src,
					     Mhtcl_Thread *dstVar));

  void (*mhtcl_Abort)	_ANSI_ARGS_((Mhtcl_Thread thread));

  More_Error (*mhtcl_Keygen) _ANSI_ARGS_((int algorithm,
					  int length,
					  More_Block block,
					  int hash1,
					  int hash2,
					  int count,
					  More_Block salt,
					  Tcl_Obj **keyVar));
  
  /* Channel interface. */

  Tcl_Channel (*mhtcl_OpenHashChannel) _ANSI_ARGS_((Mhtcl_Thread thread));
  More_Error (*mhtcl_MakeChannelHash) _ANSI_ARGS_((Tcl_Channel channel,
						   Tcl_Obj **digestVar));

  /* Miscellaneous. */

  int (*mhtcl_GetHashAlgFromObj) _ANSI_ARGS_((Tcl_Interp *	interp,
						Tcl_Obj *	objPtr,
						int *	algPtr));
  int (*mhtcl_GetKeygenAlgFromObj) _ANSI_ARGS_((Tcl_Interp *	interp,
						  Tcl_Obj *	objPtr,
						  int *algPtr));

  int (*mhtcl_HashExists)	_ANSI_ARGS_((int algorithm));
  int (*mhtcl_KeygenExists)	_ANSI_ARGS_((int algorithm));

  int (*mhtcl_IsHMACAlgo)	_ANSI_ARGS_((int algorithm));
  size_t (*mhtcl_HashBlockSize)	_ANSI_ARGS_((int algorithm));
  int (*mhtcl_KeygenUsesSalt)	_ANSI_ARGS_((int algorithm));
  int (*mhtcl_KeygenUsesCount)	_ANSI_ARGS_((int algorithm));
  int (*mhtcl_KeygenSaltSize)	_ANSI_ARGS_((int algorithm));
  int (*mhtcl_KeygenMaxKeySize)	_ANSI_ARGS_((int algorithm));
  int (*mhtcl_KeygenUsesHash)	_ANSI_ARGS_((int algorithm));

  Tcl_Obj * (*mhtcl_ListHashAlgos) _ANSI_ARGS_((void));
  Tcl_Obj * (*mhtcl_ListKeygenAlgos) _ANSI_ARGS_((void));

  int (*mhtcl_MhashVersion)	_ANSI_ARGS_((void));


} Mhtcl_StubTable;


/** ------------------------------------------------------------
 ** Stub declarations.
 ** ----------------------------------------------------------*/

#ifdef USE_MHTCL_STUB

EXTERN CONST Mhtcl_StubTable *		mhtclStubPtr;

EXTERN CONST char *	Mhtcl_InitStub	_ANSI_ARGS_((Tcl_Interp *interp,
						     char *version,
						     int exact));

#define Mhtcl_OpenHashThread		(mhtclStubPtr->mhtcl_OpenHashThread)
#define Mhtcl_OpenHMACThread		(mhtclStubPtr->mhtcl_OpenHMACThread)

#define Mhtcl_Digest			(mhtclStubPtr->mhtcl_Digest)
#define Mhtcl_Close			(mhtclStubPtr->mhtcl_Close)
#define Mhtcl_Copy			(mhtclStubPtr->mhtcl_Copy)
#define Mhtcl_Abort			(mhtclStubPtr->mhtcl_Abort)
#define Mhtcl_Keygen			(mhtclStubPtr->mhtcl_Keygen)

#define Mhtcl_OpenHashChannel		(mhtclStubPtr->mhtcl_OpenHashChannel)
#define Mhtcl_MakeChannelHash		(mhtclStubPtr->mhtcl_MakeChannelHash)

#define Mhtcl_GetHashAlgFromObj		(mhtclStubPtr->mhtcl_GetHashAlgFromObj)
#define Mhtcl_GetKeygenAlgFromObj	(mhtclStubPtr->mhtcl_GetKeygenAlgFromObj)
#define Mhtcl_HashExists		(mhtclStubPtr->mhtcl_HashExists)
#define Mhtcl_KeygenExists		(mhtclStubPtr->mhtcl_KeygenExists)
#define Mhtcl_IsHMACAlgo		(mhtclStubPtr->mhtcl_IsHMACAlgo)
#define Mhtcl_HashBlockSize		(mhtclStubPtr->mhtcl_HashBlockSize)
#define Mhtcl_KeygenUsesSalt		(mhtclStubPtr->mhtcl_KeygenUsesSalt)
#define Mhtcl_KeygenUsesCount		(mhtclStubPtr->mhtcl_KeygenUsesCount)
#define Mhtcl_KeygenSaltSize		(mhtclStubPtr->mhtcl_KeygenSaltSize)
#define Mhtcl_KeygenMaxKeySize		(mhtclStubPtr->mhtcl_KeygenMaxKeySize)
#define Mhtcl_KeygenUsesHash		(mhtclStubPtr->mhtcl_KeygenUsesHash)
#define Mhtcl_ListHashAlgos		(mhtclStubPtr->mhtcl_ListHashAlgos)
#define Mhtcl_ListKeygenAlgos		(mhtclStubPtr->mhtcl_ListKeygenAlgos)

#define Mhtcl_MhashVersion		(mhtclStubPtr->mhtcl_MhashVersion)

#endif /* USE_MHTCL_STUBS */

#endif /* __MHTCLDECLS_H */


/* end of file */
