/* mhtclStubLib.c --
   
   Part of: MHTCL
   Contents: stub library
   Date: Sun Aug 10, 2003
   
   Abstract
   
   
   
   Copyright (c) 2003, 2004 Marco Maggi
   
   This is free software; you  can redistribute it and/or modify it under
   the terms of the GNU Lesser General Public License as published by the
   Free Software  Foundation; either version  2.1 of the License,  or (at
   your option) any later version.
   
   This library  is distributed in the  hope that it will  be useful, but
   WITHOUT   ANY  WARRANTY;   without  even   the  implied   warranty  of
   MERCHANTABILITY  or FITNESS  FOR A  PARTICULAR PURPOSE.   See  the GNU
   Lesser General Public License for more details.
   
   You  should have  received a  copy of  the GNU  Lesser  General Public
   License along  with this library; if  not, write to  the Free Software
   Foundation, Inc.,  59 Temple Place,  Suite 330, Boston,  MA 02111-1307
   USA
   
*/


/** ------------------------------------------------------------
 ** Header files.
 ** ----------------------------------------------------------*/

#include "mhtcl.h"


/** ------------------------------------------------------------
 ** Defines.
 ** ----------------------------------------------------------*/

#ifndef USE_TCL_STUBS
#  define USE_TCL_STUBS
#endif
#undef USE_TCL_STUB_PROCS


/** ------------------------------------------------------------
 ** Global variables.
 ** ----------------------------------------------------------*/

#undef TCL_STORAGE_CLASS
#define TCL_STORAGE_CLASS DLLEXPORT

CONST Mhtcl_StubTable *		mhtclStubPtr;


/* Mhtcl_InitStub --

	Checks that  the correct version of  MHTCL is loaded  and that it
	supports stubs; then it initialises the stub table pointer.

   Arguments:

	interp -	the interpreter
	version -	pointer to a string representing the requested
			version
	exact -		a flag indicating if the version number should
			match exactly the one of the extension

   Results:

        Returns  a  string  representing   the  actual  version  of  the
        extension  that satisfies  the request,  or NULL  to  indicat an
        error.

   Side effects:

        Initialises the stubs table pointer.

*/

CONST char *
Mhtcl_InitStub (Tcl_Interp *interp, char *version, int exact)
{
  CONST char *	v;


  v = Tcl_PkgRequireEx(interp, "mhash", version, exact,
		       (ClientData *) &mhtclStubPtr);
  if (v == NULL)
    {
      return v;
    }

  if (mhtclStubPtr == NULL)
    {
      Tcl_SetResult(interp, "this version of mhash doesn't support stubs",
		    TCL_STATIC);
      return NULL;
    }
  return v;
}

/* end of file */
